/*
 * Copyright 2023 Nico Sonack <nsonack@herrhotzenplotz.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef GCLI_CTX_H
#define GCLI_CTX_H

#include <stdbool.h>

#include <curl/curl.h>

/* Strictly internal structure containing the gcli library context
 * data */
struct gcli_ctx {
	CURL *curl;
	char *curl_useragent;
	void *usrdata;

	char *last_error;
	char *apibase;    /* generated by a call to get_apibase */

	char *(*get_token)(struct gcli_ctx *);
	gcli_forge_type (*get_forge_type)(struct gcli_ctx *ctx);
	char *(*get_apibase)(struct gcli_ctx *);

	void (*report_progress)(bool done);
};

/* Error routine */
int gcli_error(struct gcli_ctx *ctx, char const *const fmt, ...);

char *gcli_get_apibase(struct gcli_ctx *ctx);
char *gcli_get_authheader(struct gcli_ctx *ctx);
char *gcli_get_token(struct gcli_ctx *ctx);

#endif /* GCLI_CTX_H */
